"use strict";

require("core-js/modules/es.symbol");

require("core-js/modules/es.symbol.description");

require("core-js/modules/es.symbol.iterator");

require("core-js/modules/es.array.concat");

require("core-js/modules/es.array.filter");

require("core-js/modules/es.array.find-index");

require("core-js/modules/es.array.from");

require("core-js/modules/es.array.is-array");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.map");

require("core-js/modules/es.array.slice");

require("core-js/modules/es.array.sort");

require("core-js/modules/es.date.to-iso-string");

require("core-js/modules/es.date.to-string");

require("core-js/modules/es.object.define-property");

require("core-js/modules/es.object.to-string");

require("core-js/modules/es.promise");

require("core-js/modules/es.regexp.constructor");

require("core-js/modules/es.regexp.exec");

require("core-js/modules/es.regexp.to-string");

require("core-js/modules/es.string.iterator");

require("core-js/modules/es.string.match");

require("core-js/modules/web.dom-collections.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseReleases = parseReleases;

require("regenerator-runtime/runtime");

var _semver = _interopRequireDefault(require("semver"));

var _commits = require("./commits");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var MERGE_COMMIT_PATTERN = /^Merge (remote-tracking )?branch '.+'/;
var COMMIT_MESSAGE_PATTERN = /\n+([\S\s]+)/;

function createRelease(_x, _x2, _x3, _x4, _x5) {
  return _createRelease.apply(this, arguments);
}

function _createRelease() {
  _createRelease = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(tag, previousTag, diff, remote, options) {
    var commits, merges, fixes, emptyRelease, tagPattern, tagPrefix, _ref5, date, message, breakingCount, filteredCommits;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _commits.fetchCommits)(diff, remote, options);

          case 2:
            commits = _context.sent;
            merges = commits.filter(function (commit) {
              return commit.merge;
            }).map(function (commit) {
              return commit.merge;
            });
            fixes = commits.filter(function (commit) {
              return commit.fixes;
            }).map(function (commit) {
              return {
                fixes: commit.fixes,
                commit: commit
              };
            });
            emptyRelease = merges.length === 0 && fixes.length === 0;
            tagPattern = options.tagPattern, tagPrefix = options.tagPrefix;
            _ref5 = commits[0] || {
              date: new Date().toISOString()
            }, date = _ref5.date, message = _ref5.message;
            breakingCount = commits.filter(function (c) {
              return c.breaking;
            }).length;
            filteredCommits = commits.filter(function (commit) {
              return filterCommit(commit, options, merges);
            }).sort(commitSorter(options)).slice(0, getCommitLimit(options, emptyRelease, breakingCount));
            return _context.abrupt("return", {
              tag: tag,
              title: tag || 'Unreleased',
              date: date,
              isoDate: date.slice(0, 10),
              niceDate: (0, _utils.niceDate)(date),
              commits: filteredCommits,
              merges: merges,
              fixes: fixes,
              summary: getSummary(message, options),
              major: Boolean(!tagPattern && tag && previousTag && _semver["default"].diff(tag, previousTag) === 'major'),
              href: previousTag ? remote.getCompareLink(previousTag, tag ? "".concat(tagPrefix).concat(tag) : 'HEAD') : null
            });

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _createRelease.apply(this, arguments);
}

function parseReleases(tags, remote, latestVersion, options) {
  var releases = tags.map(function (tag, index, tags) {
    var previousTag = tags[index + 1];
    var diff = previousTag ? "".concat(previousTag, "..").concat(tag) : tag;
    return createRelease(tag, previousTag, diff, remote, options);
  });

  if (latestVersion || options.unreleased) {
    var tag = latestVersion || null;
    var previousTag = tags[0];
    var diff = "".concat(previousTag, "..");
    return Promise.all([createRelease(tag, previousTag, diff, remote, options)].concat(_toConsumableArray(releases)));
  }

  return Promise.all(releases);
}

function getCommitLimit(_ref, emptyRelease, breakingCount) {
  var commitLimit = _ref.commitLimit,
      backfillLimit = _ref.backfillLimit;

  if (commitLimit === false) {
    return undefined; // Return all commits
  }

  var limit = emptyRelease ? backfillLimit : commitLimit;
  return Math.max(breakingCount, limit);
}

function filterCommit(commit, _ref2, merges) {
  var ignoreCommitPattern = _ref2.ignoreCommitPattern;

  if (commit.fixes || commit.merge) {
    // Filter out commits that already appear in fix or merge lists
    return false;
  }

  if (commit.breaking) {
    return true;
  }

  if (ignoreCommitPattern) {
    // Filter out commits that match ignoreCommitPattern
    return new RegExp(ignoreCommitPattern).test(commit.subject) === false;
  }

  if (_semver["default"].valid(commit.subject)) {
    // Filter out version commits
    return false;
  }

  if (MERGE_COMMIT_PATTERN.test(commit.subject)) {
    // Filter out merge commits
    return false;
  }

  if (merges.findIndex(function (m) {
    return m.message === commit.subject;
  }) !== -1) {
    // Filter out commits with the same message as an existing merge
    return false;
  }

  return true;
}

function getSummary(message, _ref3) {
  var releaseSummary = _ref3.releaseSummary;

  if (!message || !releaseSummary) {
    return null;
  }

  if (COMMIT_MESSAGE_PATTERN.test(message)) {
    return message.match(COMMIT_MESSAGE_PATTERN)[1];
  }

  return null;
}

function commitSorter(_ref4) {
  var sortCommits = _ref4.sortCommits;
  return function (a, b) {
    if (!a.breaking && b.breaking) return 1;
    if (a.breaking && !b.breaking) return -1;
    if (sortCommits === 'date') return new Date(a.date) - new Date(b.date);
    if (sortCommits === 'date-desc') return new Date(b.date) - new Date(a.date);
    return b.insertions + b.deletions - (a.insertions + a.deletions);
  };
}